#!/usr/bin/env python
"""au_xmltvgen_austext.py - Create a XMLTV information from AUSTEXT pages.
Version 0.1

This script requires:
  - alevtd, the teletext webserver, running.
  - A linux-supported TV card, with vbi interface.
  - A good reception for channel 7, and to have the TV-card currently tuned
    to channel 7.

To run, do au_xmltvgen_austext.py [timezone] [server host] [server port]

If no timezone is specified, it will default to +0930 (CST), except for QLD
and WA channels where the TZ is always correct
If no host is specified, it will default to localhost.
If no port is specified, it will default to 5654.

Copyright 2004 - 2005 Michael Farrell.

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.


http://www.ratemyrackmountedserver.com/~michael/projects/au_xmltvgen_austext/

For more info, please read the fine man page.
"""

import sys, httplib, re, string, time

def httpconnect(host, port):
	connection = httplib.HTTPConnection(host, port)
	try:
		connection.connect()
	except:
		print "ERROR: Couldn't connect to server."
		sys.exit(1)
	return connection

def getpage(connection, page):
	try:
		connection.request("GET", "/" + page + ".txt")
		return connection.getresponse().read()
	except:
		print "ERROR: Couldn't get page."
		sys.exit(1)

def getTvGuidePage(connection, page, channelid, timezone):
	"""
		connection: A HTTPConnection to the alevtd server
		page: The page number, a slash, then the subpage number.
		channelid: A channel ID to print.
		timezone: The timezone of the area.
	"""

	page = getpage(connection, page)
	pagearr = string.split(page, "\n")
	# find out what line the guide starts on.  This is either 11 or 12.
	if re.search("(Mon|Tues|Wednes|Thurs|Fri|Satur|Sun)day", pagearr[3]):
		pageStarts = 4
	elif re.search("(Mon|Tues|Wednes|Thurs|Fri|Satur|Sun)day", pagearr[4]):
		pageStarts = 5
	else:
		pageStarts = 6
	
	# Convert the date to a string
	now = time.localtime()
	month = str(now[1])
	if len(month) < 2:
		month = "0" + month
	day = str(now[2])
	if len(day) < 2:
		day = "0" + day
	date = str(now[0]) + month + day

	# Find the number of days this month.
	daysInMonth = 31
	if (now[1] == 2):
		# feburary
		if (now[0] == int(now[0]/4)*4):
			# the year is leap...
			daysInMonth = 29
		else:
			daysInMonth = 28
	elif (now[1] == 4) or (now[1] == 6) or (now[1] == 9) or (now[1] == 11):
		# april, june, september, november.
		daysInMonth = 30

	# If it is the first day of the month, we need the number of days in last month
	if (now[2] == 1):
		# Find the number of days last month.
		daysInLastMonth = 31
		if (now[1] == 3):
			# feburary
			if (now[0] == int(now[0]/4)*4):
				# the year is leap...
				daysInLastMonth = 29
			else:
				daysInLastMonth = 28
		elif (now[1] == 5) or (now[1] == 7) or (now[1] == 10) or (now[1] == 12):
			# april, june, september, november.
			daysInLastMonth = 30
	
	currentLine = pageStarts
	while currentLine < 20:
		# Check if the line has a time on it.
		if re.search("\.", pagearr[currentLine]):
			thisdate = date
			if (int(pagearr[currentLine][1]+pagearr[currentLine][2]) < 6) and (now[3]>6):
				# The stated time is from midnight to 6am, increment the date.
				if now[2] == daysInMonth:
					thisdate = thisdate[0:6] + "01"
					if (now[1]+1) < 10:
						thisdate[4] = "0"
					else:
						thisdate[4] = "1"
					thisdate[5] = str(now[1]+1)[1]
					if now[1]+1 == 13:
						thisdate = str(now[0]) + thisdate[4:8]
				elif now[2]+1<10:
					thisdate = thisdate[0:6] + "0" + str(now[2]+1)
				else:
					thisdate = thisdate[0:6] + str(now[2]+1)
			elif (int(pagearr[currentLine][1]+pagearr[currentLine][2]) > 5) and (now[3]<6):
				# The stated time is after 6am, and the current time is before 6am, deincrement the date.
				if now[2] == 1:
					if now[1] == 1:
						thisdate = str(now[0] - 1) + "1231"
					elif now[1]-1<10:
						thisdate = thisdate[0:4] + "0" + str(now[1]-1) + str(daysInLastMonth)
					else:
						thisdate = thisdate[0:4] + str(now[1]-1) + str(daysInLastMonth)
				elif now[2]-1<10:
					thisdate = thisdate[0:6] + "0" + str(now[2]-1)
				else:
					thisdate = thisdate[0:6] + str(now[2]-1)
			# Otherwise, leave the date be.
			
			print "<programme start=\"" + thisdate + pagearr[currentLine][1] + pagearr[currentLine][2] +  pagearr[currentLine][4] + pagearr[currentLine][5] + "00 " + timezone + "\" channel=\"" + channelid + "\">"
			programString = ""
			currentChr = 8
			while currentChr < 41:
				programString = programString + pagearr[currentLine][currentChr]
				currentChr = currentChr + 1
			programString = string.replace(programString, "& ", "&amp; ", 2);
			print "<title lang=\"en\">" + programString + "</title>"
			print "</programme>"
		currentLine = currentLine + 1
	return 1

timezone = "+0930"
hostname = "localhost"
port = 5654

# get our command line params.
if len(sys.argv) > 1:
	timezone = sys.argv[1]
	if len(sys.argv) > 2:
		hostname = sys.argv[2]
		if len(sys.argv) > 3:
			try:
				port = int(sys.argv[2])
			except:
				print "ERROR: That is not a valid port number."
				sys.exit(1)

# okay, we should have a host and port to connect to now.
connection = httpconnect(hostname, port)

# we'll start by trying to get the main page for TV-guides.

#	http://localhost:5654/650/01.html

page = getpage(connection, "650/01")

connection.close()
if re.search("videotext page not found in cache", page):
	# We haven't waited long enough.
	print "ERROR: Page 650.1 is not in alevtd's cache.  Have you waited 5 to 10 minutes after starting alevtd? Are you on channel 7?"
	sys.exit(1)
elif re.search("AUSTEXT", page):
	# we have a match!
	pass
else:
	print "ERROR: Page 650.1 doesn't look like AUSTEXT. Are you on channel 7?"
	sys.exit(1)

print """<?xml version="1.0" encoding="ISO-8859-1"?>
<!DOCTYPE tv SYSTEM "xmltv.dtd">

<tv source-info-url="http://www.seven.com.au/" source-info-name="AUSTEXT" source-data-url="http://www.seven.com.au/" generator-info-name="XMLTV - au_xmltvgen_austext" generator-info-url="http://www.ratemyrackmountedserver.com/~michael/projects/au_xmltvgen_austext/">

  <channel id="sbs.austext">
    <display-name lang="en">SBS</display-name>
  </channel>

  <channel id="abc.adl.austext">
    <display-name lang="en">ABC Adelaide</display-name>
  </channel>
  <channel id="seven.adl.austext">
    <display-name lang="en">Channel Seven</display-name>
  </channel>
  <channel id="ten.adl.austext">
    <display-name lang="en">Network TEN</display-name>
  </channel>

  <channel id="abc.bris.austext">
    <display-name lang="en">ABC Brisbane</display-name>
  </channel>
  <channel id="seven.bris.austext">
    <display-name lang="en">Channel Seven</display-name>
  </channel>
  <channel id="seven.qld.austext">
    <display-name lang="en">Seven Queensland</display-name>
  </channel>
  <channel id="ten.bris.austext">
    <display-name lang="en">Network TEN</display-name>
  </channel>

  <channel id="abc.melb.austext">
    <display-name lang="en">ABC Melbourne</display-name>
  </channel>
  <channel id="seven.melb.austext">
    <display-name lang="en">Channel Seven</display-name>
  </channel>
  <channel id="ten.melb.austext">
    <display-name lang="en">Network TEN</display-name>
  </channel>
  <channel id="prime.vic.austext">
    <display-name lang="en">PRIME</display-name>
  </channel>

  <channel id="abc.pth.austext">
    <display-name lang="en">ABC Perth</display-name>
  </channel>
  <channel id="seven.pth.austext">
    <display-name lang="en">Channel Seven</display-name>
  </channel>
  <channel id="ten.pth.austext">
    <display-name lang="en">Network TEN</display-name>
  </channel>

  <channel id="abc.syd.austext">
    <display-name lang="en">ABC Sydney</display-name>
  </channel>
  <channel id="seven.syd.austext">
    <display-name lang="en">Channel Seven</display-name>
  </channel>
  <channel id="ten.syd.austext">
    <display-name lang="en">Network TEN</display-name>
  </channel>
  <channel id="prime.nsw.austext">
    <display-name lang="en">PRIME</display-name>
  </channel>


"""

try:
  connection = httpconnect(hostname, port)

  # SBS has issues, be warned.
  getTvGuidePage(connection, '651/01', 'sbs.austext', timezone) 
  getTvGuidePage(connection, '651/02', 'sbs.austext', timezone)



  # ABC Adelaide
  getTvGuidePage(connection, '656/01', 'abc.adl.austext', timezone) 
  getTvGuidePage(connection, '656/02', 'abc.adl.austext', timezone)

  # Seven Adelaide
  getTvGuidePage(connection, '664/01', 'seven.adl.austext', timezone) 
  getTvGuidePage(connection, '664/02', 'seven.adl.austext', timezone)

  # Network Ten Adelaide
  getTvGuidePage(connection, '670/01', 'ten.adl.austext', timezone) 
  getTvGuidePage(connection, '670/02', 'ten.adl.austext', timezone)


  # ABC Sydney
  getTvGuidePage(connection, '652/01', 'abc.syd.austext', timezone) 
  getTvGuidePage(connection, '652/02', 'abc.syd.austext', timezone)

  # Seven Sydney
  getTvGuidePage(connection, '660/01', 'seven.syd.austext', timezone) 
  getTvGuidePage(connection, '660/02', 'seven.syd.austext', timezone)

  # Network Ten Syndey
  getTvGuidePage(connection, '666/01', 'ten.syd.austext', timezone) 
  getTvGuidePage(connection, '666/02', 'ten.syd.austext', timezone)

  # Prime NSW
  getTvGuidePage(connection, '657/01', 'prime.nsw.austext', timezone) 
  getTvGuidePage(connection, '657/02', 'prime.nsw.austext', timezone)


  # ABC Melbourne
  getTvGuidePage(connection, '653/01', 'abc.melb.austext', timezone) 
  getTvGuidePage(connection, '653/02', 'abc.melb.austext', timezone)

  # Seven Melbourne
  getTvGuidePage(connection, '661/01', 'seven.melb.austext', timezone) 
  getTvGuidePage(connection, '661/02', 'seven.melb.austext', timezone)

  # Network Ten Melbourne
  getTvGuidePage(connection, '667/01', 'ten.melb.austext', timezone) 
  getTvGuidePage(connection, '667/02', 'ten.melb.austext', timezone)

  # Prime VIC
  getTvGuidePage(connection, '658/01', 'prime.vic.austext', timezone) 
  getTvGuidePage(connection, '658/02', 'prime.vic.austext', timezone)

  # Queensland, their timezone doesn't have daylight savings, so it is locked.
  # ABC Brisbane
  getTvGuidePage(connection, '654/01', 'abc.bris.austext', '+1000') 
  getTvGuidePage(connection, '654/02', 'abc.bris.austext', '+1000')

  # Seven Brisbane
  getTvGuidePage(connection, '662/01', 'seven.bris.austext', '+1000') 
  getTvGuidePage(connection, '662/02', 'seven.bris.austext', '+1000')

  # Network Ten Brisbane
  getTvGuidePage(connection, '668/01', 'ten.bris.austext', '+1000') 
  getTvGuidePage(connection, '668/02', 'ten.bris.austext', '+1000')

  # Seven QLD (also Seven Central?)
  getTvGuidePage(connection, '659/01', 'seven.qld.austext', '+1000') 
  getTvGuidePage(connection, '659/02', 'seven.qld.austext', '+1000')

  # Perth also doesn't have Daylight Savings.. So they are locked in too.
  # ABC Perth
  getTvGuidePage(connection, '655/01', 'abc.pth.austext', '+0800') 
  getTvGuidePage(connection, '655/02', 'abc.pth.austext', '+0800')

  # Seven Perth
  getTvGuidePage(connection, '663/01', 'seven.pth.austext', '+0800') 
  getTvGuidePage(connection, '663/02', 'seven.pth.austext', '+0800')

  # Network Ten Perth
  getTvGuidePage(connection, '669/01', 'ten.pth.austext', '+0800') 
  getTvGuidePage(connection, '669/02', 'ten.pth.austext', '+0800')
except:
  print "ERROR: There was a problem getting the TV guides.  There is either too much data corruption, you haven't waited 5 minutes after changing to Channel 7, or AUSTEXT have changed their layout."

connection.close()


print "</tv>"

# and that's it!

